﻿'use strict';
const Gulp = require('gulp');
const Del = require('del');
const BrowserSync = require('browser-sync');
const RunSequence = require('run-sequence');
const MetalsmithBeautify = require('metalsmith-beautify');
const Karma = require('karma');
const _ = require('lodash');
const Merge2 = require('merge2');
const GulpPlumber = require('gulp-plumber');
const GulpJshint = require('gulp-jshint');
const GulpRename = require('gulp-rename');
const GulpCached = require('gulp-cached');
const GulpSass = require('gulp-sass');
const GulpAutoprefixer = require('gulp-autoprefixer');
const GulpRtlcss = require('gulp-rtlcss');
const BuildUtils = require('../util/build');
const TasksUtils = require('../util/utils');
const Config = TasksUtils.gulpConfig;
const Metalsmith = require('../util/metalsmith');
const Browserify = require('./browserify');


// Build the file without any compressions
Gulp.task('build', function (callback) {
    const buildTasks = [
        'build-lint-js',
        'build-sass',
        'build-images',
        'build-copy'
    ];

    if (!TasksUtils.isDev) {
        buildTasks.push('build-browserify');
    }
    buildTasks.push('build-static-site');

    RunSequence('build-clean', buildTasks, callback);
});

Gulp.task('build-clean', function () {
    return Del([
        Config.build.dir
    ]);
});


Gulp.task('build-lint-js', function () {
    if (!Config.build.lintJs) return;

    return Gulp.src(Config.build.lintJs.src, Config.build.lintJs.srcOpts)
        .pipe(GulpPlumber(TasksUtils.streamErrorHandler))
        .pipe(GulpCached('buildLintJs')) // only pass through files that have changed
        .pipe(GulpJshint(Config.build.lintJs.settings))
        .pipe(GulpJshint.reporter('jshint-stylish'));
});

Gulp.task('build-sass', function () {
    if (!Config.build.sass) return;

    return Gulp.src(Config.build.sass.src)
        .pipe(GulpPlumber(TasksUtils.streamErrorHandler))

        // disabled source maps because they seem to cause an issue with
        // importing bootstrap-sass
        .pipe(GulpSass(Config.build.sass.settings))
        .pipe(GulpAutoprefixer({browsers: ['last 5 version']}))

        // set name for LTR version
        .pipe(GulpRename({suffix: '-LTR'}))

        // save LTR version
        .pipe(Gulp.dest(Config.build.sass.dest))

        // rename for RTL version
        .pipe(GulpRename(function (path) {
            path.basename = path.basename.replace(/LTR$/, 'RTL');
        }))

        // convert LTR css to RTL
        .pipe(GulpRtlcss({
            autoRename: true,
            processUrls: false
        }))

        // save RTL version
        .pipe(Gulp.dest(Config.build.sass.dest))

        // reload dev server
        .pipe(BrowserSync.reload({stream: true}));
});

Gulp.task('build-images', function () {
    const bundles = TasksUtils.arrayOrEmpty(Config.build.images);
    if (!bundles) return;

    const streams = bundles.map(BuildUtils.buildImages);
    return Merge2.apply(null, streams).pipe(BrowserSync.reload({stream: true}));
});


Gulp.task('build-copy', function () {
    const bundles = TasksUtils.arrayOrEmpty(Config.build.copy);
    if (!bundles) return;

    const streams = bundles.map(BuildUtils.buildCopy);
    return Merge2.apply(null, streams).pipe(BrowserSync.reload({stream: true}));
});

Gulp.task('build-browserify', function () {
    return Browserify.browserifyTask();
});

Gulp.task('build-unit-test', function (done) {
    if (!Config.build.unitTest) return done();

    Karma.server.start(Config.build.unitTest.settings, function (exitCode) {
        if (exitCode === 0) {
            done();
        } else {
            done(new Error('Karma exited with code ' + exitCode));
        }
    });
});

Gulp.task('build-static-site', function (done) {
    Promise.resolve(Config.build.getMarkupData()).then(function (markupData) {
        markupData = _.assign(markupData, {
            baseUrl: '/build',
            staticUrl: (url) => {
                return `/build/${url}`;
            }
        });

        Metalsmith.build(markupData)
            .use(MetalsmithBeautify())
            .destination(Config.build.dir)
            .build(function (err) {
                BrowserSync.reload();
                if (err) {
                    TasksUtils.errorHandler(err);
                }
                done(err);
            });

    }).catch(done);
});
