﻿'use strict';
const Gulp = require('gulp');
const Del = require('del');
const RunSequence = require('run-sequence');
const _ = require('lodash');
const Merge2 = require('merge2');
const ReworkPluginUrl = require('rework-plugin-url');
const GulpPlumber = require('gulp-plumber');
const GulpRework = require('gulp-rework');
const GulpFilesize = require('gulp-filesize');
const GulpCleanCss = require('gulp-clean-css');
const GulpUglify = require('gulp-uglify');
const GulpImagemin = require('gulp-imagemin');
const GulpRev = require('gulp-rev');
const TasksUtils = require('../util/utils');
const Config = TasksUtils.gulpConfig;
const Metalsmith = require('../util/metalsmith');
const StaticAssets = require('../util/staticAssets');


// Compress the files
Gulp.task('compile', function (callback) {
    RunSequence('compile-clean',
        'compile-images', // needs to be before css, since we add cache busters to image urls
        'compile-css', // sometimes hits a timeout when run in parallel with other tasks
        ['compile-js', 'compile-copy'],
        'compile-static-site',
        callback);
});

Gulp.task('compile-clean', function () {
    return Del([
        Config.compile.dir
    ]);
});

Gulp.task('compile-css', function () {
    if (!Config.compile.css) return;

    return Gulp.src(Config.compile.css.src, {cwd: Config.build.dir})
        .pipe(GulpPlumber(TasksUtils.streamErrorHandler))
        .pipe(GulpRework(ReworkPluginUrl(function (url) {
            return StaticAssets.cssUrlWithRev(url);
        })))
        .pipe(GulpCleanCss(Config.compile.css.settings))
        .pipe(GulpRev())
        .pipe(Gulp.dest(Config.compile.css.dest))
        .pipe(GulpFilesize())
        .pipe(GulpRev.manifest('css-manifest.json'))
        .pipe(Gulp.dest('compile/rev'));
});

Gulp.task('compile-js', function () {
    if (!Config.compile.js) return;

    return Gulp.src(Config.compile.js.src, {cwd: Config.build.dir})
        .pipe(GulpPlumber(TasksUtils.streamErrorHandler))
        .pipe(GulpUglify(Config.compile.js.settings))
        .pipe(GulpRev())
        .pipe(Gulp.dest(Config.compile.js.dest))
        .pipe(GulpFilesize())
        .pipe(GulpRev.manifest('js-manifest.json'))
        .pipe(Gulp.dest('compile/rev'));
});

Gulp.task('compile-images', function () {
    const bundles = TasksUtils.arrayOrEmpty(Config.compile.images);
    if (!bundles) return;

    const streams = bundles.map((bundle) => {
        let stream = Gulp.src(bundle.src, bundle.srcOpts)
            .pipe(GulpPlumber(TasksUtils.streamErrorHandler))
            .pipe(GulpImagemin(bundle.settings));

        if (bundle.addRev) {
            stream = stream
                .pipe(GulpRev());
        }

        stream = stream
            .pipe(Gulp.dest(bundle.dest))
            .pipe(GulpFilesize());

        return stream;
    });

    return Merge2.apply(null, streams)
        .pipe(GulpRev.manifest('images-manifest.json'))
        .pipe(Gulp.dest('compile/rev'));
});

Gulp.task('compile-copy', function () {
    const bundles = TasksUtils.arrayOrEmpty(Config.compile.copy);
    if (!bundles) return;

    const streams = bundles.map((bundle) => {
        return Gulp.src(bundle.src, bundle.srcOpts)
            .pipe(GulpPlumber(TasksUtils.streamErrorHandler))
            .pipe(Gulp.dest(bundle.dest));
    });
    return Merge2.apply(null, streams);
});


/**
 * TODO add html minification here and in production
 .pipe(plugins.minifyHtml(config.compile.markup.minifyHtmlOptions))
 minifyHtmlOptions: {
        empty: true,
        spare: true,
        quotes: true,
        conditionals: true  // KEEP conditional internet explorer comments
      }
 */
Gulp.task('compile-static-site', function (done) {
    Promise.resolve(Config.compile.getMarkupData()).then(function (markupData) {
        markupData = _.assign(markupData, {
            baseUrl: '/compile',
            staticUrl: (url) => {
                return `/compile/${StaticAssets.filenameWithRev(url)}`;
            }
        });

        Metalsmith.build(markupData)
            .destination(Config.compile.dir)
            .build(function (err) {
                if (err) {
                    TasksUtils.errorHandler(err);
                }
                done(err);
            });
    }).catch(done);
});
