'use strict';
const Gulp = require('gulp');
const GulpUtil = require('gulp-util');
const RunSequence = require('run-sequence');
const _ = require('lodash');
const GulpAwspublish = require('gulp-awspublish');
const GulpAwspublishRouter = require('gulp-awspublish-router');
const GulpPlumber = require('gulp-plumber');

const TasksUtils = require('../util/utils');
const Config = TasksUtils.gulpConfig;


Gulp.task('deploy', function (callback) {
    RunSequence('deploy-s3',
        callback);
});

// Send the static files to S3
Gulp.task('deploy-s3', function () {

    // We'll use this as the default cache time for resources that don't have cache buster
    const DEFAULT_CACHE_TIME = 60; // 1 minute

    // We'll use this as the cache time for resources that have a cache buster
    const INFINITE_CACHE_TIME = 315360000; // 10 years

    // create a new publisher using S3 options
    // http://docs.aws.amazon.com/AWSJavaScriptSDK/latest/AWS/S3.html#constructor-property
    const publisher = GulpAwspublish.create({
        params: {
            Bucket: Config.deploy.s3.s3Bucket
        },
        region: Config.deploy.s3.s3Region
    });

    let routes = [
        // sitemap.xml is generated from metalsmith so it will be gzipped in the production version
        {
            routeMatcher: /^sitemap.xml$/,
            cacheTime: DEFAULT_CACHE_TIME,
            headers: {
                ContentEncoding: 'gzip'
            }
        },

        // JS, CSS, SVG and TTF files are gzipped, and can have cache busters
        {
            routeMatcher: /^(?:.+)-[a-f0-9]{10}\.(?:js|css|svg|ttf)$/,
            cacheTime: INFINITE_CACHE_TIME,
            headers: {
                ContentEncoding: 'gzip'
            }
        },
        {
            routeMatcher: /^(?:.+)\.(?:js|css|svg|ttf)$/,
            cacheTime: DEFAULT_CACHE_TIME,
            headers: {
                ContentEncoding: 'gzip'
            }
        },

        // Images are not gzipped, and can have cache busters
        {
            routeMatcher: /^(?:.+)-[a-f0-9]{10}\.(?:png|gif|jpg|jpeg)$/,
            cacheTime: INFINITE_CACHE_TIME
        },
        {
            routeMatcher: /^(?:.+)\.(?:png|gif|jpg|jpeg)$/,
            cacheTime: DEFAULT_CACHE_TIME
        },

        // HTML files are gzipped and cannot have cache busters
        {
            routeMatcher: /^(?:.+)\.html$/,
            cacheTime: DEFAULT_CACHE_TIME,
            headers: {
                ContentEncoding: 'gzip'
            }
        },

        // pass-through for anything that wasn't matched by routes above,
        // to be uploaded with default options
        {
            routeMatcher: /^.+$/,
            cacheTime: DEFAULT_CACHE_TIME,
            key: '$&'
        }
    ];

    if (Config.deploy.awsPublish && Config.deploy.awsPublish.extraRoutes) {
        const extraRoutes = Config.deploy.awsPublish.extraRoutes;
        routes = extraRoutes.concat(routes);

        // The cache mechanism of gulp-awspublish does not handle headers
        // So if a route has a header, such as WebsiteRedirectLocation, it will not update as expected if changed
        // The only way around it is to clear the cache for such routes
        // TODO remove when issue 73 is resolved https://github.com/pgherveou/gulp-awspublish/issues/73
        let cachedFiles = _.keys(publisher._cache);
        extraRoutes.forEach((route) => {
            if (route.headers) {
                _.remove(cachedFiles, (path) => {
                    return route.routeMatcher.test(path);
                }).forEach((removedPath) => {
                    GulpUtil.log(`Ignoring publish cache for ${removedPath} because it has headers`);
                    delete publisher._cache[removedPath];
                });
            }
        });
    }

    return Gulp.src(['**/*'], {cwd: Config.production.dir})
        .pipe(GulpPlumber(TasksUtils.streamErrorHandler))

        .pipe(GulpAwspublishRouter({
            cache: {
                // html files should not be cached
            },
            routes: routes
        }))

        .pipe(publisher.publish({
            // gulp-awspublish sets x-amz-acl to public-read by default
            // we set them to private since we use a bucket policy to control the permissions
            'x-amz-acl': 'private'
        }))

        // create a cache file to speed up consecutive uploads
        .pipe(publisher.cache())

        // print upload updates to console
        .pipe(GulpAwspublish.reporter());
});